// AMcmdShowLog.cpp: implementation of the AMcmdShowLog class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AMcmdImport.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AMcmdImport::AMcmdImport()
{

}

AMcmdImport::~AMcmdImport()
{

}

AMcmdImport::AMcmdImport(AMcmd* parent)
{
	m_parent = parent;
}

int AMcmdImport::Parse(AMcmd* parent)
{
	m_parent = parent;

	Execute();

	return 0;
}

TCHAR* AMcmdImport::GetName()
{
	return _T("IMPORT");
}

void AMcmdImport::DisplayHelp()
{
	PRINTTERM(_T("Import imports the three Access Server text files to the database\n"));
	PRINTTERM(_T("Files GUEST.TXT, HOST.TXT and PROFILE.TXT must exist in current directory\n"));
	PRINTTERM(_T("You must use LOGON before IMPORT\n"));
}

int AMcmdImport::Execute()
{
	int rc = 0;
	U32 hostid, guestid, roleid, groupid;

	// Make sure the three files exist
	TCHAR pText[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, pText);

	FILE *f;
	f = fopen(_T("GUEST.TXT"), "r");
	if (!f)
	{
		PRINTTERM(_T("No file GUEST.TXT in %s\n"), pText);
	}
	else
	{
		rc++;
	}
	fclose(f);

	f = fopen(_T("HOST.TXT"), "r");
	if (!f)
	{
		PRINTTERM(_T("No file HOST.TXT in %s\n"), pText);
	}
	else
	{
		rc++;
	}
	fclose(f);

	f = fopen(_T("PROFILE.TXT"), "r");
	if (!f)
	{
		PRINTTERM(_T("No file PROFILE.TXT in %s\n"), pText);
	}
	else
	{
		rc++;
	}

	if (rc < 3) return 1;
	rc = 0;

	// Import the Profiles ///////////////////////////////////////////////////////////
	ROLE *roles = NULL;
	NGROUP *usergroups = NULL;
	NGROUP *hostgroups = NULL;
	unsigned long nr, nu, nh;

	// Get the Roles which are named like IMP_%
	// These are from an older import and should be reused
	PRINTTERM(_T("Retrieving earlier imported roles roles from database\n"));

	ROLE p[AM_BULKSIZE];
	unsigned long n, i, ntotal;
	ntotal = 0;
	n = 0;
	theDbc.RoleQueryFirst(AM_BULKSIZE, AMCOLNAM_ROLE_NAME, AM_LIKE, _T("IMP_%"), NULL, p, &n);
	while (n > 0)
	{
		roles = (ROLE*)realloc(roles, (ntotal+n)*sizeof(ROLE));
		if (!roles)
		{
			PRINTSTRING(pText, _T("Memory exhausted after reading %d roles\n"), ntotal);
			PRINTTERM(pText);
			return(5);
		}
		memcpy(roles+ntotal, &p, n*sizeof(ROLE));
		ntotal+= n;
		theDbc.RoleQueryNext(AM_BULKSIZE, p, &n);

		PRINTSTRING(pText, _T("No of roles read: %d\n"), ntotal);
		PRINTTERM(pText);
	}
	nr = ntotal;

	// Get the NetOp user groups for reuse
	PRINTTERM(_T("Retrieving all NetOp User groups from database\n"));

	NGROUP g[AM_BULKSIZE];
	ntotal = 0;
	n = 0;
	theDbc.NetOpGroupQueryFirst(AM_GRUP, AM_BULKSIZE, AMCOLNAM_ROLE_NAME, 0, NULL, NULL, g, &n);
	while (n > 0)
	{
		usergroups = (NGROUP*)realloc(usergroups, (ntotal+n)*sizeof(NGROUP));
		if (!usergroups)
		{
			PRINTSTRING(pText, _T("Memory exhausted after reading %d user groups\n"), ntotal);
			PRINTTERM(pText);
			return(5);
		}
		memcpy(usergroups+ntotal, &g, n*sizeof(NGROUP));
		ntotal+= n;
		theDbc.NetOpGroupQueryNext(AM_GRUP, AM_BULKSIZE, g, &n);

		PRINTSTRING(pText, _T("No of user groups read: %d\n"), ntotal);
		PRINTTERM(pText);
	}
	nu = ntotal;

	// Get the Host groups for reuse
	PRINTTERM(_T("Retrieving all NetOp Host groups from database\n"));
	ntotal = 0;
	n = 0;
	theDbc.NetOpGroupQueryFirst(AM_GRUH, AM_BULKSIZE, AMCOLNAM_ROLE_NAME, 0, NULL, NULL, g, &n);
	while (n > 0)
	{
		hostgroups = (NGROUP*)realloc(hostgroups, (ntotal+n)*sizeof(NGROUP));
		if (!hostgroups)
		{
			PRINTSTRING(pText, _T("Memory exhausted after reading %d host groups\n"), ntotal);
			PRINTTERM(pText);
			return(5);
		}
		memcpy(hostgroups+ntotal, &g, n*sizeof(NGROUP));
		ntotal+= n;
		theDbc.NetOpGroupQueryNext(AM_GRUH, AM_BULKSIZE, g, &n);

		PRINTSTRING(pText, _T("No of user groups read: %d\n"), ntotal);
		PRINTTERM(pText);
	}
	nh = ntotal;

	// Walk through file PROFILE.TXT
	PRINTTERM(_T("Reading file PROFILE.TXT\n"));
	ntotal = 0;
	CString str;
	while (fgets(pText, MAX_PATH, f))
	{
		CString hgs, ugs, yn;
		int ix;
		str = pText;
		PRINTTERM(_T("%s"), str.GetBuffer(1));

		// Add the user group if it does not exist
		ix = 0;
		ix = str.Find(_T(","), ix);
		if (ix == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		hgs = str.Left(ix);
		hgs.TrimRight();
		ix++;

		BOOL found = FALSE;
		guestid = 0xFFFFFFFF;
		for (i = 0; i < nu; i++)
		{
			if (hgs.Compare(usergroups[i].name) == 0)
			{
				found = TRUE;
				guestid = usergroups[i].id;
			}
		}

		if (!found)
		{
			AMRETURN rc;
			NGROUP gp;

			STRINGCOPY(gp.name, hgs.GetBuffer(1));
			gp.enabled = 1;
			gp.id = 777;
			STRINGCOPY(gp.desc, _T("Created by import facility"));
			rc = theDbc.NetOpGroupInsert(NULL, AM_GRUP, &gp);
			if (rc != AM_SUCCESS)
			{
				PRINTSTRING(pText, _T("Error %d creating user group %s\n"), rc, gp.name);
				PRINTTERM(pText);
				return(6);
			}
			else
			{
				PRINTSTRING(pText, _T("Created user group %s Ok\n"), gp.name);
				PRINTTERM(pText);
				guestid = gp.id;
				usergroups = (NGROUP*)realloc(usergroups, (nu+1)*sizeof(NGROUP));
				if (!usergroups)
				{
					PRINTSTRING(pText, _T("Memory exhausted on %d usergroups\n"), nu);
					PRINTTERM(pText);
					return(4);
				}
				memcpy(usergroups+nu, &gp, sizeof(NGROUP));
				nu++;
			}
		}

		// Add the host group if it does not exist
		U32 iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		ugs = str.Mid(ix, iy-ix);
		ugs.TrimRight();
		ix = iy+1;

		found = FALSE;
		hostid = 0xFFFFFFFF;
		for (i = 0; i < nh; i++)
		{
			if (ugs.Compare(hostgroups[i].name) == 0)
			{
				found = TRUE;
				hostid = hostgroups[i].id;
			}
		}

		if (!found)
		{
			AMRETURN rc;
			NGROUP gp;

			STRINGCOPY(gp.name, ugs.GetBuffer(1));
			gp.enabled = 1;
			STRINGCOPY(gp.desc, _T("Created by import facility"));
			rc = theDbc.NetOpGroupInsert(NULL, AM_GRUH, &gp);
			if (rc != AM_SUCCESS)
			{
				PRINTSTRING(pText, _T("Error %d creating host group %s\n"), rc, gp.name);
				PRINTTERM(pText);
				return(7);
			}
			else
			{
				PRINTSTRING(pText, _T("Created host group %s Ok\n"), gp.name);
				PRINTTERM(pText);
				
				hostid = gp.id;
				hostgroups = (NGROUP*)realloc(hostgroups, (nh+1)*sizeof(NGROUP));
				if (!hostgroups)
				{
					PRINTSTRING(pText, _T("Memory exhausted on %d hostgroups\n"), nh);
					PRINTTERM(pText);
					return(5);
				}
				memcpy(hostgroups+nh, &gp, sizeof(NGROUP));
				nh++;
			}

		}

		// Decifre the rights flags
		CString yes = _T("Y");

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Ctrl = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Keyb = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Blnk = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Lock = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Boot = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Clip = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Send = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Recv = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Chat = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Audi = (yn.GetAt(0) == yes);

		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		yn = str.Mid(ix, iy-ix);
		yn.TrimRight();
		ix = iy+1;
		U32 Prnt = (yn.GetAt(0) == yes);

		yn = str.Mid(ix, 1);
		U32 Conf = 0;
		if (yn.IsEmpty())
		{
			PRINTTERM("Syntax error: missing confirm access flag. Set to 'Never'\n");
		}
		else
		{
			if (yn.GetAt(0) == yes) Conf = 1;
			CString loggedin = _T("L");
			if (yn.GetAt(0) == loggedin) Conf = 2;
		}

		// Does such arole exist ???
		roleid = 0xFFFFFFFF;
		ROLE rol;
		rol.Ctrl = Ctrl;
		rol.Keyb = Keyb;
		rol.Blnk = Blnk;
		rol.Lock = Lock;
		rol.Boot = Boot;
		rol.Clip = Clip;
		rol.Send = Send;
		rol.Recv = Recv;
		rol.Chat = Chat;
		rol.Audi = Audi;
		rol.Prnt = Prnt;
		rol.Conf = Conf;
		U32 index = 0xFFFFFFFF;
		i = 0;
		while (i < nr)
		{
			BOOL match = TRUE;
			if (rol.Ctrl != roles[i].Ctrl) match = FALSE;
			if (rol.Keyb != roles[i].Keyb) match = FALSE;
			if (rol.Blnk != roles[i].Blnk) match = FALSE;
			if (rol.Lock != roles[i].Lock) match = FALSE;
			if (rol.Boot != roles[i].Boot) match = FALSE;
			if (rol.Clip != roles[i].Clip) match = FALSE;
			if (rol.Send != roles[i].Send) match = FALSE;
			if (rol.Recv != roles[i].Recv) match = FALSE;
			if (rol.Chat != roles[i].Chat) match = FALSE;
			if (rol.Audi != roles[i].Audi) match = FALSE;
			if (rol.Prnt != roles[i].Prnt) match = FALSE;
			if (rol.Conf != roles[i].Conf) match = FALSE;

			if (match)
			{
				index = i;
				roleid = roles[i].id;
				break;
			}
			i++;
		}

		if (index == 0xFFFFFFFF)
		{
			// If it did not exist, create one
			STRINGCOPY(rol.desc, _T("Role created by import. You may rename it"));
			CTime now = CTime::GetCurrentTime();
			Sleep(1000); // Just to make sure that name is new
			PRINTSTRING(rol.name, _T("IMP_%s"), now.Format(_T("%Y%j%H%M%S"))); 
			rc = theDbc.RoleInsert(NULL, &rol);
			if (rc != AM_SUCCESS)
			{
				PRINTSTRING(pText, _T("Error %d creating role %s\n"), rc, rol.name);
				PRINTTERM(pText);
				return(7);
			}
			else
			{
				PRINTSTRING(pText, _T("Created role %s Ok\n"), rol.name);
				PRINTTERM(pText);

				roleid = rol.id;
				roles = (ROLE*)realloc(roles, (nr+1)*sizeof(ROLE));
				if (!roles)
				{
					PRINTSTRING(pText, _T("Memory exhausted on %d roles\n"), nr);
					PRINTTERM(pText);
					return(5);
				}
				memcpy(roles+nr, &rol, sizeof(ROLE));
				nr++;
			}
		}

		// Make the correct role assignment
		if (guestid == 0xFFFFFFFF || hostid == 0xFFFFFFFF || roleid == 0xFFFFFFFF)
		{
			PRINTSTRING(pText, _T("Error: Guestid 0x%x Hostid 0x%x Roleid 0x%x\n"), guestid, hostid, roleid);
			PRINTTERM(pText);
			return(88);
		}
		else
		{
			MAIN rass;
			rass.gtyp = AM_GRUP;
			rass.gid = guestid;
			rass.htyp = AM_GRUH;
			rass.hid = hostid;
			rass.rid = roleid;
			rass.enabled = 1;

			rc = theDbc.RoleAssignmentInsert(NULL, &rass);
			if (rc != AM_SUCCESS)
			{
				PRINTSTRING(pText, _T("Error/Warning %d creating role assignment. Maybe it already exists\n"), rc);
				PRINTTERM(pText);
				// return(87);
			}
			else
			{
				PRINTSTRING(pText, _T("Created role assignment Ok\n"));
				PRINTTERM(pText);
			}
		}
	}
	fclose(f);

	// Read file HOST.TXT /////////////////////////////////////////////////////////////////////////////////////////

	f = fopen(_T("HOST.TXT"), "r");
	if (!f)
	{
		PRINTTERM(_T("No file HOST.TXT in %s\n"), pText);
		return 1;
	}

	while (fgets(pText, MAX_PATH, f))
	{
		CString str, host, desc, hgrp;
		NHOST h;
		int ix, iy;

		str = pText;
		PRINTTERM(_T("%s"), str.GetBuffer(1));		
		ix = 0;

		// Get the host name
		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		host = str.Left(iy-ix);
		host.TrimRight();
		STRINGCOPY(h.name, host.GetBuffer(1));
		ix = iy + 1;

		// Get the host description string
		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		desc = str.Mid(ix, iy-ix);
		desc.TrimRight();
		STRINGCOPY(h.desc, desc.GetBuffer(1));
		ix = iy + 1;

		// Gte the host group (only 1 allowed in the 6.5 access server)
		iy = str.GetLength();
		if (iy-ix <= 0)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		hgrp = str.Mid(ix, iy-ix+1);
		hgrp.TrimRight();

		h.enabled = 1;
		// It becomes a lot slower if we test if it already exists every time
		rc = theDbc.NetOpHostInsert(NULL, &h);

		if (rc != AM_SUCCESS)
		{
			// An error may mean that it already existed
			U32 n = 0;
			rc = theDbc.NetOpHostQueryFirst(1, AMCOLNAM_HOST_NAME, AM_EQUAL, host.GetBuffer(1), NULL, &h, &n);
			if (n > 0)
			{
				// Ok, it was there already, use the ID
				hostid = h.id;
			}
			else
			{
				// It is not there - problems!
				hostid = 0xFFFFFFFF;
			}
		}
		else
		{
			// When we successfully created the host, the ID is returned in the structure
			hostid = h.id;
		}

		if (hostid == 0xFFFFFFFF)
		{
			PRINTTERM("Could neither find nor create host\n");
			return 55;
		}
		else
		{
			// We can now insert it in its hostgroup. All hostgroups are still in memory
			BOOL found = FALSE;
			U32 i = 0;
			U32 index = 0xFFFFFFFF;
			CString tmp;

			// First get the hostgroup id which mathces the name
			while (i < nh && !found)
			{
				tmp = hostgroups[i].name;
				if (tmp == hgrp)
				{
					index = i;
					groupid = hostgroups[i].id;
					found = TRUE;
				}
				i++;
			}

			// If it does not exist we must create it
			if (!found)
			{
				AMRETURN rc;
				NGROUP gp;

				STRINGCOPY(gp.name, hgrp.GetBuffer(1));
				gp.enabled = 1;
				STRINGCOPY(gp.desc, _T("Created by import facility"));
				rc = theDbc.NetOpGroupInsert(NULL, AM_GRUH, &gp);
				if (rc != AM_SUCCESS)
				{
					PRINTSTRING(pText, _T("Error %d creating host group %s\n"), rc, gp.name);
					PRINTTERM(pText);
					return(7);
				}
				else
				{
					PRINTSTRING(pText, _T("Created host group %s Ok\n"), gp.name);
					PRINTTERM(pText);
					
					groupid = gp.id;
					hostgroups = (NGROUP*)realloc(hostgroups, (nh+1)*sizeof(NGROUP));
					if (!hostgroups)
					{
						PRINTSTRING(pText, _T("Memory exhausted on %d hostgroups\n"), nh);
						PRINTTERM(pText);
						return(5);
					}
					memcpy(hostgroups+nh, &gp, sizeof(NGROUP));
					nh++;
				}
			}

			// Now insert the host in its hostgroup
			rc = theDbc.NetOpHostAddToGroup(hostid, groupid);

			if (rc == AM_SUCCESS)
			{
				PRINTTERM("Host added to group ok\n");
			}
			else
			{
				// Maybe it already was in that group
				NGROUP pp[AM_BULKSIZE];
				rc = theDbc.NetOpHostQueryGroups(hostid, AM_BULKSIZE, pp, &n);
				BOOL found = FALSE;
				for (i = 0; i < n; i++)
				{
					if (pp[i].id == groupid) found = TRUE;
				}

				if (!found)
				{
					PRINTTERM("Could not add host to hostgroup\n");
					return 99;
				}
				else
				{
					PRINTTERM("Already in that group\n");
				}
			}		
		}
	}
	fclose(f);

	// Read file GUEST.TXT /////////////////////////////////////////////////////////////////////////////////////////

	f = fopen(_T("GUEST.TXT"), "r");
	if (!f)
	{
		PRINTTERM(_T("No file GUEST.TXT in %s\n"), pText);
		return 1;
	}

	while (fgets(pText, MAX_PATH, f))
	{
		CString str, gust, desc, ugrp, pwd, yn;
		NUSER u;
		int ix, iy;

		str = pText;
		PRINTTERM(_T("%s"), str.GetBuffer(1));		
		ix = 0;

		// Get the guest name
		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		gust = str.Left(iy-ix);
		gust.TrimRight();
		STRINGCOPY(u.name, gust.GetBuffer(1));
		ix = iy + 1;

		// Get the guest description string
		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		desc = str.Mid(ix, iy-ix);
		desc.TrimRight();
		STRINGCOPY(u.desc, desc.GetBuffer(1));
		ix = iy + 1;

		// Get the guest (netop user) group (access profile in the 6.5 access server)
		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		ugrp = str.Mid(ix, iy-ix);
		ugrp.TrimRight();
		ix = iy + 1;

		// Get the MD5 checksum of the password and skip "0x"
		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		pwd = str.Mid(ix+2, iy-ix-2);
		pwd.TrimRight();
		ix = iy + 1;

		if (pwd.GetLength() != 32)
		{
			PRINTTERM("Warning: Password checksum does not have 32 digits\n");
			STRINGCOPY(u.password, "");
		}
		else
		{
			STRINGCOPY(u.password, pwd.GetBuffer(1));
		}


		// Skip the Administrator flag. This is now in the db security.
		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		ix = iy + 1;

		// Get the "Locked" flag
		iy = str.Find(_T(","), ix);
		if (iy == -1)
		{
			PRINTTERM("Syntax error: missing comma\n");
			return 10;
		}
		u.enabled = 0;
		CString no = "N";
		yn = str.Mid(ix, 1);
		if (yn == no) u.enabled = 1;
		ix = iy + 1;

		// Get the "Must change password" flag
		yn = str.Mid(ix, 1);
		u.mustBeChanged = 0;
		if (yn.IsEmpty())
		{
			PRINTTERM("Missign 'Must change password' flag. Set to No\n");
		}
		else
		{
			CString yes = "Y";
			if (yn == yes) u.mustBeChanged = 1;
		}

		u.nIllegal = 0; // No wrong passwords entered yet
		u.nOld = 0; // No old passwords to remember
		u.mode = 0; // No callback
		STRINGCOPY(u.callback, _T(""));

		// It becomes a lot slower if we test if it already exists every time
		rc = theDbc.NetOpUserInsert(NULL, &u);

		if (rc != AM_SUCCESS)
		{
			// An error may mean that it already existed
			U32 n = 0;
			rc = theDbc.NetOpUserQueryFirst(1, AMCOLNAM_USER_NAME, AM_EQUAL, gust.GetBuffer(1), NULL, &u, &n);
			if (n > 0)
			{
				// Ok, it was there already, use the ID
				guestid = u.id;
			}
			else
			{
				// It is not there - problems!
				guestid = 0xFFFFFFFF;
			}
		}
		else
		{
			// When we successfully created the host, the ID is returned in the structure
			guestid = u.id;
		}

		if (guestid == 0xFFFFFFFF)
		{
			PRINTTERM("Could neither find nor create guest (NetOp user)\n");
			return 55;
		}
		else
		{
			// We can now insert it in its usergroup. All usergroups are still in memory
			BOOL found = FALSE;
			U32 i = 0;
			U32 index = 0xFFFFFFFF;
			CString tmp;

			// First get the usergroup id which mathces the name
			while (i < nu && !found)
			{
				tmp = usergroups[i].name;
				if (tmp == ugrp)
				{
					index = i;
					groupid = usergroups[i].id;
					found = TRUE;
				}
				i++;
			}

			// If it does not exist we must create it
			if (!found)
			{
				AMRETURN rc;
				NGROUP gp;

				STRINGCOPY(gp.name, ugrp.GetBuffer(1));
				gp.enabled = 1;
				STRINGCOPY(gp.desc, _T("Created by import facility"));
				rc = theDbc.NetOpGroupInsert(NULL, AM_GRUP, &gp);
				if (rc != AM_SUCCESS)
				{
					PRINTSTRING(pText, _T("Error %d creating NetOp user group %s\n"), rc, gp.name);
					PRINTTERM(pText);
					return(7);
				}
				else
				{
					PRINTSTRING(pText, _T("Created NetOpuser group %s Ok\n"), gp.name);
					PRINTTERM(pText);
					
					groupid = gp.id;
					usergroups = (NGROUP*)realloc(usergroups, (nu+1)*sizeof(NGROUP));
					if (!usergroups)
					{
						PRINTSTRING(pText, _T("Memory exhausted on %d usergroups\n"), nu);
						PRINTTERM(pText);
						return(5);
					}
					memcpy(usergroups+nu, &gp, sizeof(NGROUP));
					nu++;
				}
			}

			// Now insert the host in its hostgroup
			rc = theDbc.NetOpUserAddToGroup(guestid, groupid);

			if (rc == AM_SUCCESS)
			{
				PRINTTERM("Guest (NetOp user) added to group ok\n");
			}
			else
			{
				// Maybe it already was in that group
				NGROUP pp[AM_BULKSIZE];
				rc = theDbc.NetOpUserQueryGroups(guestid, AM_BULKSIZE, pp, &n);
				BOOL found = FALSE;
				for (i = 0; i < n; i++)
				{
					if (pp[i].id == groupid) found = TRUE;
				}

				if (!found)
				{
					PRINTTERM("Could not add guest to usergroup\n");
					return 99;
				}
				else
				{
					PRINTTERM("Already in that group\n");
				}
			}		
		}
	}

	if (roles) free(roles);
	if (usergroups) free(usergroups);
	if (hostgroups) free(hostgroups);
	return rc;
}

void AMcmdImport::Error(int err)
{
	switch (err)
	{
	case AM_ERR_NOT_LOGGED_ON:
		PRINTTERM (_T("You are not logged on\n"));
		break;
	default:
		PRINTTERM (_T("Error %d\n"), err);
		break;
	}
}
