#ifndef DBIODBC_INCLUDED
#define DBIODBC_INCLUDED
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the DBIODBC_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// DBIODBC_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.

#ifdef WIN32
#ifdef DBIODBC_EXPORTS
#define DBIODBC_API __declspec(dllexport)
#else
#define DBIODBC_API __declspec(dllimport)
#endif
#else
// UNIX stuff
#endif

#include <sqlext.h>

class DBIODBC_API DBIodbc {
public:
	AMRETURN QueryRelease();
	TCHAR m_lastStmt[10000];
	AMRETURN PrepareSQL(TCHAR*, U32);
	AMRETURN CanTransact();
	AMRETURN ExecSQL(TCHAR *stmt);
	DBIodbc(void);
	~DBIodbc(void);

	AMBOOL IsLoggedOn();

	AMRETURN Insert
		(TCHAR *table, 
		U32 ncols,
		TCHAR *cols,
		AMVALS *p);

	AMRETURN QuerySingle(
		TCHAR *table,				// name of table
		U32 ncols,					// number of columns to fetch from
		TCHAR *colnames,			// comma-separated list of column names
		AMVALS *p,					// list of values to fill in, see comment below
		TCHAR *colnamWhere,			// name of column for where condition or NULL
		AMVALS *pWhere,				// single AMVALS with value for where x=y
		U32 *n);

	AMRETURN QueryFirst(
		TCHAR *table,				// name of table
		U32 ncols,					// number of columns to fetch from
		TCHAR *colnames,			// comma-separated list of column names
		AMVALS *p,					// list of values to fill in, see comment below
		U32 cond,					// AM_EQUAL, AM_LIKE, AM_GT or AM_LT
		TCHAR *colnamWhere,			// name of column for where condition or NULL
		AMVALS *pWhere,				// single AMVALS with value for where condition
		TCHAR *orderby,				// name of column to order by or NULL
		U32 num,					// Max number of records to return
		U32 *n);					// number of records actually returned (<= num)

	AMRETURN QueryNext (AMVALS *p,	// list of values to fill in, see comment below
		U32 num,					// Max number of records to return
		U32 *n);					// number of records actually returned (<= num)

	AMRETURN QueryAlt(				// Query while first stmt handle is open
		TCHAR *table,				// name of table
		U32 ncols,					// number of columns to fetch from
		TCHAR *colnames,			// comma-separated list of column names
		AMVALS *p,					// list of values to fill in, see comment below
		U32 cond,					// AM_EQUAL, AM_LIKE, AM_GT or AM_LT
		TCHAR *colnamWhere,			// name of column for where condition or NULL
		AMVALS *pWhere,				// single AMVALS with value for where condition
		TCHAR *orderby,				// name of column to order by or NULL
		U32 num,					// Max number of records to return
		U32 *n);					// number of records actually returned (<= num)

	AMRETURN Update(TCHAR *table,	// name of table
		U32 ncols,					// number of columns to fetch from
		TCHAR *colnames,			// comma-separated list of column names
		AMVALS *p,					// list of values to fill in, see comment below
		TCHAR *colnamWhere,			// name of column for "where =" clause
		AMVALS *pWhere);			// single AMVALS with value for where condition

	AMRETURN Delete(TCHAR *table,	// name of table
		TCHAR *colnamWhere,			// name of column for where condition
		AMVALS *pWhere);			// single AMVALS with value for where condition

	AMRETURN QueryNextID(
		TCHAR *table,				// name of table
		TCHAR *col,					// name of column
		U32 *n);					// next unused ID
	
	AMRETURN Logoff();
	AMRETURN Logon(TCHAR*, U32, TCHAR*, U32, TCHAR*, U32);
	AMRETURN CheckTables(U32*);
    AMRETURN CheckColumn(TCHAR*, TCHAR*, U32*);
	AMRETURN WriteLog(TCHAR*);
	AMRETURN WriteDebug(TCHAR*);
	AMRETURN WriteDebugReport(U32);
	AMRETURN WriteErr(TCHAR*);
	AMRETURN SetDebug(AMBOOL);
	AMBOOL   GetDebug();
	AMRETURN SetLogCallback(void*, void*);
protected:

private:
	void AddInt(TCHAR* txt, U32 val);
	void AddTchar(TCHAR* txt, TCHAR* val);
	TCHAR* PosAfterNextNull(TCHAR *p);
	TCHAR* TruncAtComma(TCHAR* p);
	U32 m_ncols;
	TCHAR* Trunc(TCHAR* txt, U32 length);
	AMRETURN GetDiagRec(SQLSMALLINT type);
	AMRETURN allocEnvHandle();
	AMRETURN freeEnvHandle();
	AMRETURN allocDbcHandle();
	AMRETURN freeDbcHandle();
	AMRETURN allocStmtHandle();
	AMRETURN freeStmtHandle();
	AMRETURN allocStmtHandleAlt();
	AMRETURN freeStmtHandleAlt();

	SQLHANDLE m_envHandle;
	SQLHANDLE m_dbcHandle;
	SQLHANDLE m_stmHandle;
	SQLHANDLE m_stmHandleAlt;
	
	AMBOOL m_isLoggedOn;
	AMBOOL m_debug;
	AM_MSG_FUNC m_msgFunc;
	void *m_this;
};

extern DBIODBC_API int nDBIodbc;

DBIODBC_API int fnDBIodbc(void);

#else
#endif